//test of hypotheses
//z_test
function pv=z_test(Mu0,aver,vr,n,side,alpha)
    //pv... p-value
    //Mu0 ... expectation via H0
    //aver ... sample average
    //vr ... population variance
    //n ... sample length    
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if argn(2)==5 then
        alpha=0.05;
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end

    T=(aver-Mu0)/(sqrt(vr)/sqrt(n));
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'o' then
        pv=2*min(P,Q);
        Alpha=alpha/2;
        z_alpha=cdfnor("X",0,1,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        z_alpha=cdfnor("X",0,1,1-alpha,alpha);
    case 'l' then
        z_alpha=cdfnor("X",0,1,alpha,1-alpha);
        pv=P;
    end
endfunction

//t test
function pv=t_test(Mu0,aver,vr,n,side,alpha)
    //pv... p-value
    //Mu0 ... expectation via H0
    //aver ... sample average
    //vr ... sample variance
    //n ... sample length
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if argn(2)==5 then
        alpha=0.05;
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    
    T=(aver-Mu0)/(sqrt(vr)/sqrt(n));
    [P,Q]=cdft("PQ",T,n-1)
    select side
    case 'o' then
        pv=2*min(P,Q)
        Alpha=alpha/2;
        t_alpha=cdft("T",n-1,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        t_alpha=cdft("T",n-1,1-alpha,alpha);
    case 'l' then
        pv=P;
        t_alpha=cdft("T",n-1,alpha,1-alpha);
    end
endfunction

//test of ratio
function pv=prop_test(p0,p,n,side,alpha)
    //pv... p-value
    //p0 ... probab. via H0
    //p ... probab. from sample
    //n ... sample length
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if p>1 then
        prav=p/n;
    else
        prav=p;
    end
    if argn(2)==4 then
        alpha=0.05;
    end
    
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    T=(prav-p0)/sqrt(p0*(1-p0)/n)
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'o' then
        pv=2*min(P,Q);
        Alpha=alpha/2;
        z_alpha=cdfnor("X",0,1,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        z_alpha=cdfnor("X",0,1,1-alpha,alpha);
    case 'l' then
        pv=P;
        z_alpha=cdfnor("X",0,1,alpha,1-alpha);
    end
endfunction

//test of variance
function pv=var_test(vr0,vr,n,side,alpha)
    //pv... p-value
    //n ... sample length
    //vr ... sample variance
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if argn(2)==4 then
        alpha=0.05;        
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    T=((n-1)/vr0)*vr;
    [P,Q]=cdfchi("PQ",T,n-1)
    select side
    case 'o' then
        pv=2*min(P,Q);
        Alpha=alpha/2;            
        hm_chi_alpha_2=cdfchi("X",n-1,1-Alpha,Alpha);
        dm_chi_alpha_2=cdfchi("X",n-1,Alpha,1-Alpha);
        chi_alpha=[dm_chi_alpha_2,hm_chi_alpha_2]
    case 'p' then
        pv=Q;
        hm_chi_alpha=cdfchi("X",n-1,1-alpha,alpha);
        chi_alpha=[0,hm_chi_alpha]
    case 'l' then
        pv=P;
        dm_chi_alpha=cdfchi("X",n-1,alpha,1-alpha);
        chi_alpha=[dm_chi_alpha,%inf]
    end  
endfunction

// ------------

//parametric tests for two samples
function pv=z_test_2(aver_1,vr_1,n1,aver_2,vr_2,n2,side,alpha)
    //pv ... p-value
    //aver_1 ... sample average 1
    //vr_1 ... sample variance 1
    //n1 .. sample length
    //aver_2 ... sample average 2
    //vr_2 ... sample variance 2
    //n2 ... sample length 2
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if argn(2)==7 then
        alpha=0.05;
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    T=(aver_1-aver_2)/sqrt((vr_1/n1)+(vr_2/n2));
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'o' then
        pv=2*min(P,Q);
        Alpha=alpha/2;
        z_alpha=cdfnor("X",0,1,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        z_alpha=cdfnor("X",0,1,1-alpha,alpha);
    case 'l' then
        pv=P;
        z_alpha=cdfnor("X",0,1,alpha,1-alpha);
    end
endfunction

//t test - equal variances
function pv=t_test_2s(aver_1,vr_1,n1,aver_2,vr_2, n2,side,alpha)
    //pv ... p-value
    //aver_1 ... sample average 1
    //vr_1 ... sample variance 1
    //n1 .. sample length 1
    //aver_2 ... sample average 2
    //vr_2 ... sample variance č. 2
    //n2 ... sample length č. 2
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if argn(2)==7 then
        alpha=0.05;       
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    
    Df=n1+n2-2;
    sp=((n1-1)*vr_1+(n2-1)*vr_2)/(Df);   
    T=(aver_1-aver_2)/(sqrt(sp)*sqrt((1/n1)+(1/n2)));
    [P,Q]=cdft("PQ",T,Df)
    select side
    case 'o' then
        pv=2*min(P,Q)
        Alpha=alpha/2;
        t_alpha=cdft("T",Df,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        t_alpha=cdft("T",Df,1-alpha,alpha);
    case 'l' then
        pv=P;
        t_alpha=cdft("T",Df,alpha,1-alpha);
    end
endfunction

//t test different variances
function pv=t_test_2n(aver_1,vr_1,n1,aver_2,vr_2, n2,side,alpha)
    //pv ... p-value
    //aver_1 ... sample average 1
    //vr_1 ... sample variance 1
    //n1 .. sample length 1
    //aver_2 ... sample average 2
    //vr_2 ... sample variance 2
    //n2 ... sample length 2
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    if argn(2)==7 then
        alpha=0.05;       
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    
    R1_n1=vr_1/n1;
    R2_n2=vr_2/n2;   
    T=(aver_1-aver_2)/(sqrt(R1_n1+R2_n2));
    Df=round((R1_n1+R2_n2)^2/((R1_n1^2/(n1-1))+(R2_n2^2/(n2-1))));
    [P,Q]=cdft("PQ",T,Df)
    select side
    case 'o' then
        pv=2*min(P,Q)
        Alpha=alpha/2;
        t_alpha=cdft("T",Df,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        t_alpha=cdft("T",Df,1-alpha,alpha);
    case 'l' then
        pv=P;
        t_alpha=cdft("T",Df,alpha,1-alpha);
    end
endfunction

//t test paired
function pv=t_test_2p(sample1,sample2,side,alpha)
    //pv ... p-value
    //sample1 ... values of the sample 1
    //sample2 ... values of the sample 2
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence
    vektor1=sample1(:)';
    vektor2=sample2(:)';
    if argn(2)==3 then
        alpha=0.05;       
    end
    if alpha>0.5 then
        disp('alpha je kritická hodnota, musí být tedy < 0.5');
        return
    end
    n=max(size(vektor1));
    x_d=sum(vektor1-vektor2)/n;
    s_d=sqrt((sum((vektor1-vektor2)^2)/n)-x_d^2);
    T=(x_d/s_d)*sqrt(n-1);
    Df=n-1;
    [P,Q]=cdft("PQ",T,Df)
    select side
    case 'o' then
        pv=2*min(P,Q)
        Alpha=alpha/2;
        t_alpha=cdft("T",Df,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        t_alpha=cdft("T",Df,1-alpha,alpha);
    case 'l' then
        pv=P;
        t_alpha=cdft("T",Df,alpha,1-alpha);
    end
endfunction

//test of ratio
function pv=prop_test_2(p1,n1,p2,n2,side,alpha)
    //pv... p-value
    //p1 ... sample probability 1
    //n1 ... sample length  1
    //p2 ... sample probability 2
    //n2 ... pocet prvků výběru 2
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence

    if p1>1 then
        prav1=p1/n1;
    else
        prav1=p1;
    end
    if p2>1 then
        prav2=p2/n2;
    else
        prav2=p2;
    end
    if argn(2)==5 then
        alpha=0.05;
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end

    pp=(prav1*n1+prav2*n2)/(n1+n2);
    T=(prav1-prav2)/(sqrt(pp*(1-pp))*sqrt(1/n1+1/n2));
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'o' then
        pv=2*min(P,Q);
        Alpha=alpha/2;
        z_alpha=cdfnor("X",0,1,1-Alpha,Alpha);
    case 'p' then
        pv=Q;
        z_alpha=cdfnor("X",0,1,1-alpha,alpha);
    case 'l' then
        pv=P;
        z_alpha=cdfnor("X",0,1,alpha,1-alpha);
    end
endfunction

//test two variances
function pv=var_test_2(vr_S1,n1,vr_S2,n2,side,alpha)
    //pv... p-value
    //vr_S1 ... sample variance 1
    //n1 ... sample length 1
    //vr_S2 ... sample variance 2
    //n1 ... sample length 2
    //side ... typ IS ('l'=left, 'p'=right, 'o' both)
    //alpha ... confidence

    if argn(2)==5 then
        alpha=0.05;
    end
    if alpha>0.5 then
        disp('alpha < 0.5');
        return
    end
    if vr_S1>vr_S2 then
        T=vr_S1/vr_S2;
    else
        T=vr_S2/vr_S1;
    end
    Df1=n1-1;
    Df2=n2-1;
    [P,Q]=cdff("PQ",T,Df1,Df2);

    select side
    case 'o' then
        pv=2*min(P,Q);
        Alpha=alpha/2;            
        hm_f_alpha_2=cdff("F",Df1,Df2,1-Alpha,Alpha);
        dm_f_alpha_2=cdff("F",Df1,Df2,Alpha,1-Alpha);
        f_alpha=[dm_f_alpha_2,hm_f_alpha_2]
    case 'p' then
        pv=Q;
        hm_f_alpha=cdff("F",Df1,Df2,1-alpha,alpha);
        f_alpha=[0,hm_f_alpha]
    case 'l' then
        pv=P;
        dm_f_alpha=cdff("F",Df1,Df2,alpha,1-alpha);
        f_alpha=[dm_f_alpha,%inf]
    end  
endfunction


  
